#ifndef HARDWARE_H
#define HARDWARE_H

// define the oscillator frequency for the __delay_ms function
#define _XTAL_FREQ 20000000

// include MCU specific things
#include <htc.h>

// for standard types like uint8_t
#include <stdint.h>

// init the microcontroller hardware
void hardwareInit();

// global interrupt enable 
void enableInterrupt();

// set control and data lines of the LCD
void lcdSet(uint8_t lcdE, uint8_t lcdRS, uint8_t data);

// 1 kHz interrupt callback
void onTick();


// ports

extern uint8_t g_portA;
extern uint8_t g_portD;

// push buttons
#define BUTTON1 (1 << 1)  // RE1
#define BUTTON2 (1 << 2)  // RE2

// output/control to 4004 CPU
#define CPU_TEST (1 << 4)  // RD4
#define CPU_RESET (1 << 5)  // RD5
#define CPU_D0 (1 << 0)  // RA0
#define CPU_D1 (1 << 1)  // RA1
#define CPU_D2 (1 << 2)  // RA2
#define CPU_D3 (1 << 3)  // RA3
#define CPU_ENABLE_DATA_OUT (1 << 4)  // RA4
#define CPU_PHI1 (1 << 5)  // RA5
#define CPU_PHI2 (1 << 6)  // RA6

// LED and buttons
#define setRedLed(on) PORTE = on ? 0 : 1
#define isButton1Pressed() ((PORTE & BUTTON1) == 0)
#define isButton2Pressed() ((PORTE & BUTTON2) == 0)

// input from 4004 CPU
#define CPU_D0_IN (1 << 0)  // RB0
#define CPU_D1_IN (1 << 1)  // RB1
#define CPU_D2_IN (1 << 2)  // RB2
#define CPU_D3_IN (1 << 3)  // RB3
#define CPU_CM_ROM (1 << 4)  // RB4
#define CPU_SYNC (1 << 5)  // RB5
#define CPU_CM_RAM_0 (1 << 0)  // RD0
#define CPU_CM_RAM_1 (1 << 1)  // RD1
#define CPU_CM_RAM_2 (1 << 2)  // RD2
#define CPU_CM_RAM_3 (1 << 3)  // RD3

// helper macros
#define setPortBit(portBuffer, port, on, b) { if (on) portBuffer |= b; else portBuffer &= ~b; port = portBuffer; }
#define setPortABit(on, b) setPortBit(g_portA, PORTA, on, b)
#define setPortDBit(on, b) setPortBit(g_portD, PORTD, on, b)

// CPU port access macros
#define setCpuTest(on) setPortDBit(on, CPU_TEST)
#define setCpuReset(on) setPortDBit(!(on), CPU_RESET)
#define setCpuData(d) { g_portA &= ~(0xf); g_portA |= (((d) & 0xf) ^ 0xf); PORTA = g_portA; }
#define setCpuEnableDataOut(on) setPortABit(on, CPU_ENABLE_DATA_OUT)
#define setPhi1(on) setPortABit(on, CPU_PHI1)
#define setPhi2(on) setPortABit(on, CPU_PHI2)
#define getCpuData() (PORTB & 0xf)
#define isCpuCmRom() ((PORTB & CPU_CM_ROM) == 0)
#define isCpuSync() ((PORTB & CPU_SYNC) > 0)
#define getCpuCmRam() ((PORTD & 0xf) ^ 0xf)

#endif
