#include "hardware.h"

// LCD display signals
#define LCD_RS_BIT (1 << 0)  // RC0
#define LCD_E_BIT (1 << 1)  // RC1

#define lcdSet(lcdE, lcdRS, data)\
{\
	uint8_t d2 = 0;\
	uint8_t control = 0;\
	uint8_t d = 0;\
	if (lcdE) control |= LCD_E_BIT;\
	if (lcdRS) control |= LCD_RS_BIT;\
	d = data & 0xf;\
	if (d & (1 << 0)) d2 |= 1 << 5;\
	if (d & (1 << 1)) d2 |= 1 << 4;\
	if (d & (1 << 2)) d2 |= 1 << 3;\
	if (d & (1 << 3)) d2 |= 1 << 2;\
	PORTC = d2 | control;\
}

#define lcdFunctionSetNibble(data)\
{\
	lcdSet(0, 0, data);\
	__delay_ms(1);\
	lcdSet(1, 0, data);\
	__delay_ms(1);\
	lcdSet(0, 0, 0);\
	__delay_ms(1);\
}

static void lcdFunctionSet(uint8_t data)
{
	lcdFunctionSetNibble(data >> 4);
	lcdFunctionSetNibble(data);
}

#define lcdRamSetNibble(data)\
{\
	lcdSet(0, 1, data);\
	__delay_ms(1);\
	lcdSet(1, 1, data);\
	__delay_ms(1);\
	lcdSet(0, 0, data);\
	__delay_ms(1);\
}

static void lcdRamSet(uint8_t data)
{
	lcdRamSetNibble(data >> 4);
	lcdRamSetNibble(data);
}

void lcdInit()
{
	// init 4 bit mode
	__delay_ms(50);
	lcdFunctionSetNibble(3);  // function set
	__delay_ms(50);
	lcdFunctionSetNibble(3);  // function set
	__delay_ms(50);
	lcdFunctionSetNibble(3);  // function set
	__delay_ms(50);
	lcdFunctionSetNibble(2);  // function set
	__delay_ms(50);

	// init display
	__delay_ms(50);
	lcdFunctionSet(0b00101000);  // function set
	__delay_ms(50);
	lcdFunctionSet(0b00101000);  // function set
	__delay_ms(50);
	lcdFunctionSet(0b00001100);  // display on/off
	__delay_ms(50);
	lcdFunctionSet(0b00000001);  // display clear
	__delay_ms(2);
	lcdFunctionSet(0b00000110);  // entry mode set
	__delay_ms(2);
}

void lcdClear()
{
	lcdFunctionSet(0b00000001);  // display clear
}

static void setPosition(uint8_t x, uint8_t y)
{
	uint8_t position = x;
	if (y) position += 0x40;
	lcdFunctionSet(0b10000000 | position);  // set DDRAM address
}

void lcdCursorOn()
{
	lcdFunctionSet(0b00001111);  // display on/off
	__delay_ms(50);
}

void setChar(uint8_t x, uint8_t y, uint8_t c)
{
	setPosition(x, y);
	lcdRamSet(c);  // set character
}

void drawTextLine(uint8_t y, const char* text)
{
	uint8_t x = 0;
	while (*text) {
		setChar(x++, y, *text);
		text++;
	}
	while (x < 16) {
		setChar(x++, y, ' ');
	}
}

void drawNumber(uint8_t x, uint8_t y, uint8_t number)
{
	setChar(x, y, (number / 10) + '0');
	setChar(x + 1, y, (number % 10) + '0');
}
